use super::Device2 as Device;
use ash::{prelude::*, vk};

pub struct Semaphore {
    device: Device,
    inner: vk::Semaphore,
}

impl Semaphore {
    pub fn new(device: Device) -> VkResult<Self> {
        let mut type_info = vk::SemaphoreTypeCreateInfo::default()
            .semaphore_type(vk::SemaphoreType::BINARY);
        let create_info =
            vk::SemaphoreCreateInfo::default().push_next(&mut type_info);
        let inner =
            unsafe { device.dev().create_semaphore(&create_info, None)? };

        Ok(Self { device, inner })
    }
    pub fn new_timeline(device: Device, value: u64) -> VkResult<Self> {
        let mut type_info = vk::SemaphoreTypeCreateInfo::default()
            .semaphore_type(vk::SemaphoreType::TIMELINE)
            .initial_value(value);
        let create_info =
            vk::SemaphoreCreateInfo::default().push_next(&mut type_info);
        let inner =
            unsafe { device.dev().create_semaphore(&create_info, None)? };

        Ok(Self { device, inner })
    }
    pub fn semaphore(&self) -> vk::Semaphore {
        self.inner
    }
}

impl Drop for Semaphore {
    fn drop(&mut self) {
        unsafe {
            self.device.dev().destroy_semaphore(self.inner, None);
        }
    }
}
